*****************************************
*
; BouQLder QLash
; draw the initial cave
; (C) W. Lenerz 2018
*
*****************************************

	section basic

	include 'dev1_basic_boulder_asm_keys'
	include 'dev8_keys_qlv'
	include 'dev8_keys_err'
	include 'dev8_keys_sys'
	include 'dev8_keys_con'
	include 'dev8_keys_sbasic'
	include 'dev8_keys_qdos_io'
	include 'DEV8_keys_qdos_sms'

	xdef	bd_draw
	xdef	ani_tbl32
	xdef	ani_tbl16

	xref	drw_status
; this draws the initial cave

; the caves are made up of a 22 rows * 40 cols grid =880 squares
; the array is DIMmed as DIM cv_data%(22,41)
; cols 0 and 41 are always 0
; cols 1 and 40 are always steel wall (except for intermissions)
; row  0 is always 0
; rows 1 and 22 are always steel wall

;******************************
;
; BD_DRAW wark_area
;
*******************************

errbp	moveq	#err.ipar,d0
	rts

bd_draw move.w	sb.gtlin,a2
	jsr	(a2)			; get long integer
	subq.w	#1,d3			; there must be only 1
	bne.s	errbp			; but there wasn't
	addq.l	#4,sb_arthp(a6) 	; adjust maths stack
	move.l	(a6,a1.l),a5		; pointer to work space
	move.l	scr_off(a5),a0		; screen address of 1st visible row
	move.l	a0,a3
	move.l	array(a5),a4
	sub.l	#rowsize+2,a4		; point to border
	move.w	sp_xsiz(a5),d2		; sprite x size in pixels
	mulu	scr_byt(a5),d2		; nbr of bytes for one line of a sprite
	move.w	sp_ysiz(a5),d3		; nbr of lines in 1 sprite/square
	mulu	scr_inc(a5),d3		; * increase for 1 screen line
	moveq	#0,d4
	move.w	scr_spinc(a5),d4	; screen line increase, minus 1 sprite
	moveq.l #nbrrows,d5		; there are 22 rows to draw
	move.w	endcol(a5),d6
	move.w	startcol(a5),d1
	sub.w	d1,d6			; nbr of columns to treat
	moveq	#41,d0			; tot cols + 2 elements
	sub.w	d6,d0			; cols to jump over
	swap	d6			; keep nbr of columns to treat
	add.w	d0,d0			; word sized!
	move.w	d0,d6			; offset from one row to the other
	sub.w	d6,a4			; see at ani_add
	add.w	d1,d1			; int array is word sized
	add.w	d1,a4			: point to start col in array
	move.w	scr_lop(a5),d7		; nbr of LWs to copy 1 sprite line

; d0 volatile
; d1 volatile
; d2 nbr bytes per sprite line
; d3 nbr of bytes increase for one row of squares (=16 screen lines)
; d4 increase for one screen line minus sprite x size in pixels
; d5 counters row | col
; d6 nbr of cols to treat | offset between array rows
; d7 nbr of move.l per sprite line
; a0 current pointer to screen address for sprite to be drawn
; a1 volatile
; a2 volatile
; a3 pointer to start of row in screen mem
; a4 array
; a5 mem pointer

ani_row swap	d5			; row | col
	swap	d6
	move.w	d6,d5			; there are this many col to draw per row
	swap	d6
	sub.l	d2,a0			; preset col (out of wdw (see ani_col)
ani_add add.w	d6,a4			; jump over 0th & elmnts at end of previous row + colstart
ani_col add.l	d2,a0			; advance to next col
	move.w	(a4)+,d0		; get type
	blt.s	neg_sp			; future moving object
spchk	cmp.w	#7,d0			; empty space?
	beq.s	ani_lp1 		; ... yes, do nothing
	add.w	d0,d0
	move.l	spr_tbl(a5),a1		; sprite table for this screen mode
	move.w	(a1,d0.w),d0		; index into sprites
	beq.s	ani_lp1 		; none
	move.l	spr_gen(a5),a1		; point to sprites
	add.l	d0,a1			; this is the one we want
	move.l	a0,a2
	moveq	#15,d0
sprow	move.w	d7,d1
spcol	move.l	(a1)+,(a2)+
	dbf	d1,spcol		; copy sprite data directly into screen mem
	add.l	d4,a2			; next row of sprite in screen
	dbf	d0,sprow

ani_lp1 dbf	d5,ani_col		; treat all game area cols
	swap	d5
	add.l	d3,a3			; move down one row
	move.l	a3,a0
	dbf	d5,ani_row		; treat all game area rows
status	bsr	drw_status		; draw status bar
	clr.l	d0
	rts

neg_sp	neg.w	d0
	bra.s	spchk

; table for animated sprites (modes 32/33)
ani_tbl32
	dc.w	sp32_rfrd		; 0
	dc.w	sp32_bldr		; 1
	dc.w	sp32_inbx		; 2
	dc.w	sp32_dirt		; 3
	dc.w	sp32_swal		; 4
	dc.w	sp32_wall		; 5
	dc.w	sp32_wall		; 6 expanding wall
	dc.w	0			; 7 space
	dc.w	sp32_wall		; 8 magic wall looks like normal wall
	dc.w	sp32_diam		; 9
	dc.w	sp32_amoe		;10
	dc.w	sp32_ffly		;11
	dc.w	sp32_ffly		;12
	dc.w	sp32_ffly		;13
	dc.w	sp32_ffly		;14
	dc.w	sp32_bfly		;15
	dc.w	sp32_bfly		;16
	dc.w	sp32_bfly		;17
	dc.w	sp32_bfly		;18
	dc.w	sp32_expl		;19
	dc.w	sp32_expl+512		;20
	dc.w	sp32_expl+1024
	dc.w	sp32_expl+1536
	dc.w	sp32_expl+2048
	dc.w	sp32_expl
	dc.w	sp32_expl+512
	dc.w	sp32_expl+1024
	dc.w	sp32_expl+1536
	dc.w	sp32_expl+2048
	dc.w	0
	dc.w	sp32_bldr		; 30
	dc.w	sp32_bldr		; 31
	dc.w	sp32_bldr		; 32
	dc.w	0
	dc.w	sp32_diam		; 34
	dc.w	sp32_diam		; 35
	dc.w	sp32_diam		; 36
	dc.w	sp32_rblk		; 37
	dc.w	sp32_rtap		; 38
	dc.w	sp32_rbth		; 39
	dc.w	sp32_rlft		; 40
	dc.w	sp32_rrit		; 41
	dc.w	sp32_mwal		; 42 magic wall when it is on
	dc.w	sp32_mwal+512
	dc.w	sp32_mwal+1024
	dc.w	sp32_mwal+1536

; table for animated sprites (mode 16)
ani_tbl16
	dc.w	sp16_rfrd	     ; 0
	dc.w	sp16_bldr		; 2
	dc.w	sp16_inbx		; 4
	dc.w	sp16_dirt		; 6
	dc.w	sp16_swal		; 8
	dc.w	sp16_wall		; 10
	dc.w	sp16_wall		; expanding wall
	dc.w	0			; 7 space
	dc.w	sp16_wall		; magic wall looks like normal wall
	dc.w	sp16_diam
	dc.w	sp16_amoe		; <---- mustn't be 0! hence the 2 offset
	dc.w	sp16_ffly
	dc.w	sp16_ffly
	dc.w	sp16_ffly
	dc.w	sp16_ffly
	dc.w	sp16_bfly
	dc.w	sp16_bfly
	dc.w	sp16_bfly
	dc.w	sp16_bfly
	dc.w	sp16_expl
	dc.w	sp16_expl+256
	dc.w	sp16_expl+512
	dc.w	sp16_expl+768
	dc.w	sp16_expl+1024
	dc.w	sp16_expl
	dc.w	sp16_expl+256
	dc.w	sp16_expl+512
	dc.w	sp16_expl+768
	dc.w	sp16_expl+1024
	dc.w	0		       ; spare
	dc.w	sp16_bldr	       ;
	dc.w	sp16_bldr	       ;
	dc.w	sp16_bldr	       ;
	dc.w	0
	dc.w	sp16_diam
	dc.w	sp16_diam
	dc.w	sp16_diam
	dc.w	sp16_rblk		; 37
	dc.w	sp16_rtap		; 38
	dc.w	sp16_rbth
	dc.w	sp16_rlft
	dc.w	sp16_rrit
	dc.w	sp16_mwal		; 42 magic wall when it is on
	dc.w	sp16_mwal+256
	dc.w	sp16_mwal+512
	dc.w	sp16_mwal+768

	end
