****************************************************
*
; BouQLder QLash
; scan loop action routine :  check and move butterfly
; (C) W. Lenerz 2018
*
*****************************************************


	section action

	include 'dev1_basic_boulder_asm_keys'

	xdef	bfly_act

	xref	drw_spr
	xref	drw_blk
	xref	exp_set


; d0   r   0 if OK, 1 if diggeridoo is dead, -1 if dying
; d1 c	s  key(s) hit
; d2 c	s  the object type
; d4 c	p  sprite animation counter
; d5 c	p  loop counter (row|col)
; d7 c	p  nbr of ticks betwen full scans
; a0 c	p  channel ID
; a3 c	p  timer address
; a4 c	p  ptr to NEXT element in array
; a5 c	p  workspace
; a6 c	p  basic ptr

; all other regs are volatile and may be used at will

; See the comments for fireflies. Butterflies behave the same except thet:
; - they try to turn right instead of left
; - they exlode to diamonds
	
explode lea	-2(a4),a1		; fly is centre of the explosion
	moveq	#expldia,d2
	bra	exp_set 		; do explosion and return

bfly_act
	moveq	#15,d1			; preset basic fly (facing left)

; FIRST check if fly should cause explosion
; a fly explodes if an element to its left, right, up, down is either diggeridoo
; or an amoeba
flies	moveq	#amoeba,d3
	move.w	(a4),d0 		; element to the right of fly
	beq.s	explode 		; is diggeridoo
	cmp.w	d3,d0			; is it amoeba?
	beq.s	explode 		; ... yes
	move.w	-4(a4),d0		; element to the left of fly
	beq.s	explode 		; is diggeridoo
	cmp.w	d3,d0			; is it amoeba?
	beq.s	explode 		; ... yes
	move.w	rowsize-2(a4),d0	; element below fly
	beq.s	explode 		; is diggeridoo
	cmp.w	d3,d0			; is it amoeba?
	beq.s	explode 		; ... yes
	move.w	-rowsize-2(a4),d0	; element above fly
	beq.s	explode 		; is diggeridoo
	cmp.w	d3,d0			; is it amoeba?
	beq.s	explode 		; ... yes
; D1 is the basic fly, i.e. one facing left
no_expl move.w	d1,d6
	move.l	a4,a2			: keep current position ...
	subq.l	#2,a2			; ... of this fly
	move.w	d2,d0			; type/direction of fly
	move.l	d5,d2
	sub.w	d1,d0			; which direction is fly facing?
	bne.s	fly_dur 		; fly isn't facing left->
fl_left 				; fly is facing left, try right turn = move down on screen
	cmp.w	#space,-rowsize(a2)	; where I'd move to, but is it free?
	bne.s	fl_fwd			; ... no->
	sub.w	#rowsize,a2		; element in array to set
	add.l	#$10000,d2		; new y position if I move up
	addq.w	#fly_up,d1		; new type of fly (up)
	bra	set_fly 		; delete this fly, set new fly
; left facing fly, try to move forward (i.e. left)
fl_fwd	cmp.w	#space,-2(a2)		; where I'd move to, but is it free?
	bne.s	fl_rt			; ... no ->
	subq.w	#2,a2			; element to set
	addq.w	#1,d2			: new col
	bra	set_fly 		; set fly
; left facing fly, turn left (i.e. down), but don't change position
fl_rt	addq.w	#fly_dwn,d1		; this fly is now one that faces down
	bra.s	fl_set

; here fly doesn't face left, perhaps it faces down     12
fly_dur subq.w	#1,d0			; is fly facing down?
	bne.s	fly_ur			; ... no, must be up or right ->
; facing down, try right turn (move to left)
	cmp.w	#space,-2(a2)		; where I'd move to, but is it free?
	bne.s	fd_fwd			; ... no ->
	subq.l	#2,a2			; element to set
	addq.w	#1,d2			: new col
	bra.s	set_fly 		; delete this fly, set new fly
; down facing fly, try to move forward (i.e. down)
fd_fwd	cmp.w	#space,rowsize(a2)	; this is where I'd move to, but is it free?
	bne.s	fd_rt			; ... no ->
	add.w	#rowsize,a2
	sub.l	#$10000,d2		; new y position if I move down
	addq.w	#fly_dwn,d1		; down fly
	neg.w	d1			;
	bra.s	set_fly 		; set fly
; down facing fly, turn left (i.e. face right), but don't paint
fd_rt	addq.w	#fly_rit,d1

fl_set	move.w	d1,-2(a4)		; this fly is now one that faces left
done	moveq	#0,d0
	rts

; here fly didn't face down, perhaps it faces up
fly_ur	subq.w	#1,d0			; is fly facing up?
	bne.s	fly_r			; no, must be right
					; facing up, try right turn (move to right)
	cmp.w	#space,(a4)		; where I'd move to, but is it free?
	bne.s	fu_fwd			; ... no ->
	move.l	a4,a2
	subq.w	#1,d2			; new column
	addq.w	#3,d1			; right facing fly
	neg.w	d1			; new fly at right, type 14 (right) negative
	bra.s	set_fly 		; delete this fly, set new fly

; up facing fly, try to move forward (i.e. up)
fu_fwd	cmp.w	#space,-rowsize(a2)	; where I'd move to, but is it free?
	bne.s	fl_set			; ... no , so turn left (face left)
	sub.w	#rowsize,a2
	add.l	#$10000,d2		; new row up
	addq.w	#fly_up,d1		; up fly
	bra.s	set_fly 		; set fly
; up facing fly, turn left (i.e. face leftt), but don't paint

; here fly faces right
fly_r					; facing right, try right turn (move down)
	cmp.w	#space,rowsize(a2)	; where I'd move to, but is it free?
	bne.s	fr_fwd			; ... no ->
	add.w	#rowsize,a2
	sub.l	#$10000,d2		; new y position if I move down
	addq.w	#fly_dwn,d1		; down fly
	neg.w	d1			;
	bra.s	set_fly 		; set fly
; right facing fly, try to move forward (i.e. right)
fr_fwd	cmp.w	#space,(a4)		    ; this is where I'd move to, but is it free?
	bne.s	fr_rt			; ... no
	subq.w	#1,d2
	move.l	a4,a2
	addq.w	#3,d1
	neg.w	d1
	bra.s	set_fly 		;
; right facing fly, turn left (i.e. face up), but don't change position now
fr_rt	addq.w	#fly_up,d1		; this fly is now one that faces left
	bra.s	fl_set

; setting the fly : put space at my current position, redraw at new position
; d1 new fly type
; d2	   (row|col) of new position
; d5 c	p  (row|col) of current position
; a2 new position in array
; a4 position of NEXT element in array
fly_reg reg	d1/d2/a2

set_fly move.w	#7,-2(a4)	: signal space in current position
	movem.l d1/d2/a2,-(a7)	; keep new position
	bsr	drw_blk 	; erase sprite at current position (d5)
	movem.l (a7)+,d1/d2/a2	; new position
	move.w	d1,(a2) 	; fly at new position
	move.w	d6,d0		; sprite number
	move.l	d5,d6		; keep this reg
	move.l	d2,d5
	bsr	drw_spr 	; draw it there now
	move.l	d6,d5
	tst.l	d0
	rts








	end
