****************************************************
*
; BouQLder QLash
; scan loop action routine : check for diamond fall/roll/hit
; (C) W. Lenerz 2018
*
*****************************************************

	section basic

	include 'dev1_basic_boulder_asm_keys'
	include 'dev1_basic_boulder_asm_snd_keys'

	xdef	diafall
	xdef	dia4rol
	xdef	chkdia
	xdef	inc_dia

	xref	sound
	xref	drw_blk
	xref	drw_spr
	xref	exp_set
	xref	new_spc
	xref	inc_score

; d0   r   0 if OK, 1 if diggeridoo is dead, -1 if dying
; d1 c	s  key(s) hit
; d2 c	s  the object type
; d4 c	p  sprite animation counter
; d5 c	p  loop counter (row|col)
; d7 c	p  nbr of ticks betwen full scans
; a0 c	p  channel ID
; a3 c	p  timer address
; a4 c	p  ptr to NEXT element in array
; a5 c	p  workspace
; a6 c	p  basic ptr

; all other regs are volatile and may be used at will

; falling diamond
diafall move.w	rowsize-2(a4),d3	; element below the current one
	bge.s	bf2
	neg.w	d3
bf2	move.l	d5,d6			; keep curent row / col counter
	cmp.w	#space,d3		: keep falling?
	bne.s	ck_roll 		; ... no
	moveq	#-falldia,d0		; ... yes, set element below as falling diamond
bf_drw	move.w	d0,rowsize-2(a4)	; set element below
bf_drw2 sub.l	#$10000,d5		; one row down	(REVERSE! d5 = loop counter)
	moveq	#diamond,d0
	bsr	drw_spr 		; draw sprite at new location
	move.l	d6,d5
bf_spce bsr	new_spc 		; space at current location
	moveq	#0,d0
	rts
; check whether diamond landed on an element with which is interacts
; i.e. stationary diamond,stationary boulder, brick wall, magic wall, diggeridoo
ck_roll move.l	#snd_diai,d1
	bsr	sound
	move.w	d3,d0
	beq.s	you_die 		: diamond falls on diggeridoo - rip
	subq.w	#boulder,d0		; is object below a boulder
	beq.s	bf_ckrl 		; ... yes, this might roll
	subq.w	#brickwl-boulder,d0	; is object below a brickwall?
	beq.s	bf_ckrl 		; ... yes, this might roll
	subq.w	#3,d0			; is object below magic wall?
	beq.s	bf_mgwl 		; ... yes, this might fall through
	subq.w	#1,d0			; is object below a doamond
	beq.s	bf_ckrl 		; ... yes, this might roll

	subq.w	#1,d0			; amoeba or anything in between?
	ble.s	no_actn 		; ... no
	subq.w	#8,d0			; flies?
	ble.s	flies			; ... no

no_actn moveq	#diamond,d0		; ... no, so this is now a simple diamond
	move.w	d0,-2(a4)		; show it
	moveq	#0,d0
	rts

; here diamond landed on a roll-inducing element - can it roll off it?
bf_ckrl 				; first check for roll left
	cmp.w	#space,-4(a4)		; square to the left is empty?
	bne.s	bf_ckrr 		; ... no, check roll right
	cmp.w	#space,rowsize-4(a4)	; space left and down empty?
	bne.s	bf_ckrr 		; ... no, check right
	moveq	#-falldia,d0		; ... yes, diamond will roll left
	move.w	d0,-4(a4)		; square the left is falling diamond
** ** should I draw the diamond to the left here?
	bra.s	bf_spce 		; draw space at current location & leave

bf_ckrr moveq	#diamond,d0		; preset no roll
	cmp.w	#space,(a4)		; is square to the right empty?
	bne.s	no_actn 		; ... no, done
	cmp.w	#space,rowsize(a4)	; is square right and down empty?
	bne.s	no_actn 		; ... no, done
	moveq	#-falldia,d0		; ... yes, diamond will roll right
	move.w	d0,(a4) 		; square to the right is falling diamond
** ** should I draw the diamond to the right here?
	bra.s	bf_spce 		; draw space at current location & leave


; Here diamond landed on a magic wall.
; If the wall is not active but can be activated, activate it.
; If the wall is active, and there is a space beneath it, the diamond turns
; into a boulder, else it just disappears.
; If the wall is not active, the diamond just lands on there.
bf_mgwl tst.w	mw_on(a5)		; is wall on?
	blt.s	no_actn 		; ... no and it can't be turned on, done
	beq.s	mgw_son 		1 ... no but it will be turned on now

; magic wall is on. If there is space underneath, turn diamond into boulder
mgw_on	cmp.w	#space,rowsize+rowsize-2(a4) ; is there a space underneath wall?
	bne	bf_spce 		; no space underneath, diamond just disappears
	moveq	#-fallbdr,d0		; turns to falling boulder (!)
	move.w	d0,rowsize+rowsize-2(a4)
	bra	bf_spce 		; and draw a space at current location
mgw_son move.w	mw_time(a5),mw_on(a5)	; magic wall is now on
	bra.s	mgw_on			; so see what happens

you_die move.w	#space,-2(a4)		; diamond becomes space
expl	moveq	#explosion,d2		; deadly explosion
	lea	rowsize-2(a4),a1	; centre of the explosion
	bra	exp_set 		; explosion & death

; here it landed on a fly
flies	addq.w	#4,d0
	ble.s	expl
	moveq	#expldia,d2
	lea	rowsize-2(a4),a1	; centre of the explosion
	bra	exp_set 		; explosion & death


;------------------------------------------------------------------
; check whether a diamond at (a1) should roll, if so mark it
; this marks the respective squares, but does NOT redraw them!!!
; condition codes : Z if diamond should roll
; d0  s
; d1  s
;------------------------------------------------------------------
chkdia	cmp.w	#diamond,(a1)		; is it a diamond?
	bne.s	no_roll 		; no
dia4rol move.w	rowsize(a1),d1		; square underneath = surface on which bldr rests
	sub.w	#boulder,d1		; is boulder ?
	beq.s	may_rol 		: ... yes
	subq.w	#4,d1			; is brickwall ?
	beq.s	may_rol 		; ... yes
	subq.w	#4,d1
	bne.s	no_roll 		; is not boulder, so diamond can't roll
may_rol 				; check for roll left first
	cmp.w	#space,-2(a1)		; is left square free space?
	bne.s	c4r_rit 		; ... no, check right
	cmp.w	#space,-2+rowsize(a1)	; is square below that free space?
	bne.s	c4r_rit 		; ... no, try right
rol_lft move.w	#newspc,(a1)		; square here is now empty
	move.w	#falldia,-2(a1) 	; diamond will fall
	moveq	#0,d0
no_roll rts
c4r_rit cmp.w	#space,2(a1)		; is right square space?
	bne.s	no_roll 		; ... no, done
	cmp.w	#space,2+rowsize(a1)	; is square below that space?
	bne.s	no_roll 		; ... no, done
	move.w	#-newspc,(a1)		; square here is now empty
	move.w	#falldia,2(a1)		; diamond will fall
	moveq	#0,d0
	rts

;-------------------------------------------------------------------
; increase diamond count, show increased nmber & score, make sound
;-------------------------------------------------------------------
inc_dia moveq	#0,d0
	move.w	dia_val(a5),d0
	add.l	d0,points(a5)		; new nbr of points
	move.w	dia_got(a5),d0
	addq.w	#1,d0			; one more diamond got
	move.w	d0,dia_got(a5)
	cmp.w	dia_req(a5),d0		; enough diamonds to leave?
	bne	inc_score		; ... no, just make sound
	tst.b	exit_ok(a5)
	bne	inc_score
	move.b	#1,exit_ok(a5)		; ... yes, I can leave now
	move.l	a2,-(a7)
	move.l	exitloc(a5),a2
	move.w	#aniexit,(a2)		; this is now an animated exit bax
	move.l	(a7)+,a2
	move.b	#1,white(a5)		; ... draw white spaces
	move.w	dia_bva(a5),dia_val(a5) ; new diamond value
	bra	inc_score		; make sound & return

	end
