****************************************************
*
; BouQLder QLash
; scan loop action routine : check and move firefly
; (C) W. Lenerz 2018
*
*****************************************************





	section action

	include 'dev1_basic_boulder_asm_keys'

	xdef	ffly_act

	xref	new_spc
	xref	drw_spr
	xref	drw_blk
	xref	exp_set

; d1   r   0 if OK, 1 if diggeridoo is dead, -1 if dying
; d2 c s   the object (fly)
; d4 c	p  sprite animation counter
; d5 c	p  loop counter (row|col)
; d7 c	p  loop delay | nbr of ticks before rescan for movement
; a0 c	p  channel ID
; a1   s
; a2   s
; a3 c	p  timer address
; a4 c	p  ptr to NEXT element in array
; a5 c	p  workspace
; a6 c	p  basic ptr
; Check whether a fly can move and if so, to where it moves, and move it.
; A fly always has a direction in which it is facing. When trying to move,
; a fly first tries to make a left turn (with respect to the direction it is
; facing) and move one square in the new direction. If it can't move there,
; the fly tries to move forward (with respect to the direction it is facing)
; and move one square forwars. If it can't do that, the fly turns right but
; does not move a suqare into the new direction.
; (  Left/forward = immediate move, right = move only at next scan).
; What "left, forward, right" mean in absolute terms with respect to the screen
; depends on the direction the fly is facing.

; This here must :
; first check if explosion should occur
; check what kind of move is possible,
; set the new value in the new element in the array where fly moved to **
; draw the fly at the new position
; set a space to the old element in the array
; blank the square where the fly used to be

	
explode lea	-2(a4),a1		; fly is centre of the explosion
	moveq	#explosion,d2
	bra	exp_set 		; do explosion and return

ffly_act
	moveq	#11,d1

; FIRST check if fly should cause explosion
; a fly explodes if an element to its left, right, up, down is either diggeriddo
; or an amoeba
flies	moveq	#amoeba,d3
	move.w	(a4),d0 		; element to the right of fly
	beq.s	explode 		; is diggeriddo
	cmp.w	d3,d0			; is it amoeba?
	beq.s	explode 		; ... yes
	move.w	-4(a4),d0		; element to the left of fly
	beq.s	explode 		; is diggeriddo
	cmp.w	d3,d0			; is it amoeba?
	beq.s	explode 		; ... yes
	move.w	rowsize-2(a4),d0	; element below fly
	beq.s	explode 		; is diggeriddo
	cmp.w	d3,d0			; is it amoeba?
	beq.s	explode 		; ... yes
	move.w	-rowsize-2(a4),d0	; element above fly
	beq.s	explode 		; is diggeriddo
	cmp.w	d3,d0			; is it amoeba?
	beq.s	explode 		; ... yes

no_expl move.w	d1,d6
	move.l	a4,a2			: keep current position ...
	subq.l	#2,a2			; ... of this fly
	move.w	d2,d0			; type/direction of fly
	move.l	d5,d2
	sub.w	d1,d0			; which direction is fly facing?
	bne.s	fly_dur 		; fly isn't facing left->

fl_left 				; fly is facing left, try left turn = move down on screen
	cmp.w	#space,rowsize(a2)	; where I'd move to, but is it free?
	bne.s	fl_fwd			; ... no->
** remember here and at the others: exceptionnally, to move UP I must add 1
** to move down deduct one same for left right
	sub.l	#$10000,d2		; new y position if I move down
	add.w	#rowsize,a2		; array element
	addq.w	#1,d1			; down	sprite
	neg.w	d1			; new fly at row below, type 12 (down) negative
	bra	set_fly 		; delete this fly, set new fly
; left facing fly, try to move forward (i.e. left)
fl_fwd	cmp.w	#space,-2(a2)		; where I'd move to, but is it free?
	bne.s	fl_rt			; ... no ->
	subq.w	#2,a2			; element to set
	addq.w	#1,d2			: new col
	bra	set_fly 		; set fly
; left facing fly, turn right (i.e. up), but don't change position
fl_rt	addq.w	#2,d1			; this fly is now one that faces up
	bra.s	fd_rt

; here fly doesn't face left, perhaps it faces down     12
fly_dur subq.w	#1,d0			; is fly facing down?
	bne.s	fly_ur			; ... no, must be up or right ->
; facing down, try left turn (move to right)
	cmp.w	#space,(a4)		; where I'd move to, but is it free?
	bne.s	fd_fwd			; ... no ->
	move.l	a4,a2
	subq.w	#1,d2			; new column
	addq.w	#3,d1			; right facing fly
	neg.w	d1			; new fly at right, type 14 (right) negative
	bra.s	set_fly 		; delete this fly, set new fly
; down facing fly, try to move forward (i.e. down)
fd_fwd	cmp.w	#space,rowsize(a2)	; this is where I'd move to, but is it free?
	bne.s	fd_rt			; ... no ->
	add.w	#rowsize,a2
	sub.l	#$10000,d2		; new y position if I move down
	addq.w	#1,d1			; down fly
	neg.w	d1			;
	bra.s	set_fly 		; set fly
; down facing fly, turn right (i.e. face left), but don't paint
fd_rt	move.w	d1,-2(a4)		; this fly is now one that faces left
done	moveq	#0,d0
	rts

; here fly didn't face down, perhaps it faces up 13
fly_ur	subq.w	#1,d0			; is fly facing up?
	bne.s	fly_r			; no, must be right
debug nop					 ; facing up, try left turn (move to left)
	cmp.w	#space,-2(a2)		; where I'd move to, but is it free?
	bne.s	fu_fwd			; ... no ->
	subq.l	#2,a2			; element to set
	addq.w	#1,d2			: new col
	bra.s	set_fly 		; delete this fly, set new fly
; up facing fly, try to move forward (i.e. up)
fu_fwd	cmp.w	#space,-rowsize(a2)	; where I'd move to, but is it free?
	bne.s	fu_rt			; ... no
	sub.w	#rowsize,a2
	add.l	#$10000,d2		; new row up
	addq.w	#2,d1			; up fly
	bra.s	set_fly 		; set fly
; up facing fly, turn right (i.e. face right), but don't paint
fu_rt	addq.w	#3,d1			; right facing fly
	bra.s	fd_rt

; here fly faces right
fly_r					; facing right, try left turn (move up)
	cmp.w	#space,-rowsize(a2)	; where I'd move to, but is it free?
	bne.s	fr_fwd			; ... no ->
	sub.w	#rowsize,a2		; element in array to set
	add.l	#$10000,d2		; new y position if I move up
	addq.w	#2,d1			; new type of fly (up)
	bra.s	set_fly 		; set fly
; right facing fly, try to move forward (i.e. right)
fr_fwd	cmp.w	#space,(a4)		    ; this is where I'd move to, but is it free?
	bne.s	fr_rt			; ... no
	subq.w	#1,d2
	move.l	a4,a2
	addq.w	#3,d1
	neg.w	d1
	bra.s	set_fly 		; swap d2 & set fly
; right facing fly, turn right (i.e. face down), but don't change position now
fr_rt	addq.w	#1,d1			; this fly is now one that faces left
	bra.s	fd_rt

; setting the fly : put space at my current position, redraw at new position
; d1 new fly type
; d2	   (row|col) of new position
; d5 c	p  (row|col) of current position
; a2 new position in array
; a4 position of NEXT element in array
fly_reg reg	d1/d2/a2
set_fly
	movem.l d1/d2/a2,-(a7)		; keep new position
	bsr	new_spc 		; erase sprite at current position (d5)
	movem.l (a7)+,d1/d2/a2		; new position
	move.w	d1,(a2) 		; fly at new position
	move.w	d6,d0			; sprite number
	move.l	d5,d6			; keep this reg
	move.l	d2,d5			; new position
	bsr	drw_spr 		; draw it there now
	move.l	d6,d5
	tst.l	d0
	rts

	end
